package vc.thinker.absctacts.protocolproxy.protocol;

import cn.hutool.core.thread.ThreadUtil;
import com.google.common.collect.Maps;
import lombok.extern.slf4j.Slf4j;
import vc.thinker.absctacts.protocolproxy.dto.MeterInfo;
import vc.thinker.mqtt.proxyclient.entity.down.PlateFormMeterInfo;

import javax.annotation.PostConstruct;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
import java.util.stream.Collectors;

/**
 * 第三方协议
 *
 * @author HeTongHao
 * @since 2022/8/19 16:20
 */
@Slf4j
public abstract class AbstractThirdPartyProtocolSyncMeterInfo extends AbstractThirdPartyProtocol implements IThirdPartyProtocolSyncMeterInfo {
    private final Map<String, MeterInfo> idMeterMap = Maps.newConcurrentMap();

    @PostConstruct
    protected void initMeterList() {
        ThreadUtil.execute(() -> {
            int retryCount = 0;
            do {
                try {
                    pullMeterList();
                    return;
                } catch (TimeoutException e) {
                    log.warn("[{}]协议拉取仪表超时，第{}次重试中...", protocolType().getName(), ++retryCount);
                    try {
                        TimeUnit.SECONDS.sleep(10);
                    } catch (InterruptedException ex) {
                        ex.printStackTrace();
                    }
                }
            } while (true);
        });
    }

    public synchronized Collection<MeterInfo> getMetes() {
        return idMeterMap.values();
    }

    @Override
    public boolean pullMeterList() throws TimeoutException {
        try {
            List<PlateFormMeterInfo> plateFormMeterInfos = thirdPartyProxyClientService.pullMeterList(protocolType().getCode());
            Map<String, MeterInfo> idMeterMap = plateFormMeterInfos.stream()
                    .map(plateFormMeterInfo -> MeterInfo
                            .builder()
                            .id(plateFormMeterInfo.getId())
                            .code(plateFormMeterInfo.getCode())
                            .build()
                    )
                    .collect(Collectors.toMap(MeterInfo::getId, meterInfo -> meterInfo));
            synchronized (this) {
                this.idMeterMap.clear();
                this.idMeterMap.putAll(idMeterMap);
                return true;
            }
        } catch (TimeoutException e) {
            throw e;
        } catch (Exception e) {
            log.error("[{}]协议拉取仪表异常:{}", protocolType().getName(), e.getMessage());
            e.printStackTrace();
            return false;
        }
    }

    @Override
    public void putMeter(MeterInfo meterInfo) {
        this.idMeterMap.put(meterInfo.getId(), meterInfo);
    }

    @Override
    public void deleteMeter(String id) {
        this.idMeterMap.remove(id);
    }
}
